@@ -15,10 +15,10 @@ OBS_PROJECT_BETA=home:ivaradi:beta
 OBS_PACKAGE=nextcloud-desktop
 
 if test "${DRONE_TARGET_BRANCH}" = "stable-2.6"; then
-    UBUNTU_DISTRIBUTIONS="bionic focal groovy hirsute"
+    UBUNTU_DISTRIBUTIONS="bionic focal hirsute impish"
     DEBIAN_DISTRIBUTIONS="buster stretch testing"
 else
-    UBUNTU_DISTRIBUTIONS="focal groovy hirsute"
+    UBUNTU_DISTRIBUTIONS="focal hirsute impish"
     DEBIAN_DISTRIBUTIONS="testing"
 fi
 
@@ -88,7 +88,7 @@ for distribution in ${UBUNTU_DISTRIBUTIONS} ${DEBIAN_DISTRIBUTIONS}; do
     dpkg-genchanges -S -sa > "../nextcloud-desktop_${fullver}_source.changes"
 
     if test -f ~/.has_ppa_keys; then
-        debsign -k7D14AA7B -S
+        debsign -k2265D8767D14AA7B -S
     fi
 done
 cd ..
