@@ -31,13 +31,23 @@ HRESULT SetHKCRRegistryKeyAndValue(PCWSTR pszSubKey, PCWSTR pszValueName, PCWSTR
 
     if (SUCCEEDED(hr))
     {
+        DWORD cbData;
+        const BYTE * lpData;
+
         if (pszData)
         {
             // Set the specified value of the key.
-            DWORD cbData = lstrlen(pszData) * sizeof(*pszData);
-            hr = HRESULT_FROM_WIN32(RegSetValueEx(hKey, pszValueName, 0,
-                REG_SZ, reinterpret_cast<const BYTE *>(pszData), cbData));
+            cbData = lstrlen(pszData) * sizeof(*pszData);
+            lpData = reinterpret_cast<const BYTE *>(pszData);
         }
+        else
+        {
+            cbData = 0;
+            lpData = NULL;
+        }
+
+        hr = HRESULT_FROM_WIN32(RegSetValueEx(hKey, pszValueName, 0,
+            REG_SZ, lpData, cbData));
 
         RegCloseKey(hKey);
     }
@@ -88,6 +98,11 @@ HRESULT NCContextMenuRegHandler::RegisterInprocServer(PCWSTR pszModule, const CL
     {
         hr = SetHKCRRegistryKeyAndValue(szSubkey, nullptr, pszFriendlyName);
 
+        // Create the HKCR\CLSID\{<CLSID>}\ContextMenuOptIn subkey.
+        if (SUCCEEDED(hr)) {
+            hr = SetHKCRRegistryKeyAndValue(szSubkey, L"ContextMenuOptIn", NULL);
+        }
+
         // Create the HKCR\CLSID\{<CLSID>}\InprocServer32 key.
         if (SUCCEEDED(hr))
         {
