@@ -144,6 +144,9 @@ QByteArray makeChecksumHeader(const QByteArray &checksumType, const QByteArray &
 
 QByteArray findBestChecksum(const QByteArray &_checksums)
 {
+    if (_checksums.isEmpty()) {
+        return {};
+    }
     const auto checksums = QString::fromUtf8(_checksums);
     int i = 0;
     // The order of the searches here defines the preference ordering.
@@ -162,7 +165,7 @@ QByteArray findBestChecksum(const QByteArray &_checksums)
         return _checksums.mid(i, end - i);
     }
     qCWarning(lcChecksums) << "Failed to parse" << _checksums;
-    return QByteArray();
+    return {};
 }
 
 bool parseChecksumHeader(const QByteArray &header, QByteArray *type, QByteArray *checksum)
@@ -361,11 +364,11 @@ void ValidateChecksumHeader::slotChecksumCalculated(const QByteArray &checksumTy
     const QByteArray &checksum)
 {
     if (checksumType != _expectedChecksumType) {
-        emit validationFailed(tr("The checksum header contained an unknown checksum type '%1'").arg(QString::fromLatin1(_expectedChecksumType)));
+        emit validationFailed(tr("The checksum header contained an unknown checksum type \"%1\"").arg(QString::fromLatin1(_expectedChecksumType)));
         return;
     }
     if (checksum != _expectedChecksum) {
-        emit validationFailed(tr("The downloaded file does not match the checksum, it will be resumed. '%1' != '%2'").arg(QString::fromUtf8(_expectedChecksum), QString::fromUtf8(checksum)));
+        emit validationFailed(tr(R"(The downloaded file does not match the checksum, it will be resumed. "%1" != "%2")").arg(QString::fromUtf8(_expectedChecksum), QString::fromUtf8(checksum)));
         return;
     }
     emit validated(checksumType, checksum);
