@@ -128,12 +128,19 @@ namespace FileSystem {
      * Returns the file system used at the given path.
      */
     QString fileSystemForPath(const QString &path);
-#endif
 
-    QByteArray OCSYNC_EXPORT calcMd5(const QString &fileName);
-    QByteArray OCSYNC_EXPORT calcSha1(const QString &fileName);
-#ifdef ZLIB_FOUND
-    QByteArray OCSYNC_EXPORT calcAdler32(const QString &fileName);
+    /*
+     * This function takes a path and converts it to a UNC representation of the
+     * string. That means that it prepends a \\?\ (unless already UNC) and converts
+     * all slashes to backslashes.
+     *
+     * Note the following:
+     *  - The string must be absolute.
+     *  - it needs to contain a drive character to be a valid UNC
+     *  - A conversion is only done if the path len is larger than 245. Otherwise
+     *    the windows API functions work with the normal "unixoid" representation too.
+     */
+    QString OCSYNC_EXPORT pathtoUNC(const QString &str);
 #endif
 
     /**
@@ -146,51 +153,15 @@ namespace FileSystem {
      */
     bool OCSYNC_EXPORT isLnkFile(const QString &filename);
 
+    /**
+     * Returns whether the file is an exclude file (contains patterns to exclude from sync)
+     */
+    bool OCSYNC_EXPORT isExcludeFile(const QString &filename);
+
     /**
      * Returns whether the file is a junction (windows only)
      */
     bool OCSYNC_EXPORT isJunction(const QString &filename);
-
-    /*
-     * This function takes a path and converts it to a UNC representation of the
-     * string. That means that it prepends a \\?\ (unless already UNC) and converts
-     * all slashes to backslashes.
-     *
-     * Note the following:
-     *  - The string must be absolute.
-     *  - it needs to contain a drive character to be a valid UNC
-     *  - A conversion is only done if the path len is larger than 245. Otherwise
-     *    the windows API functions work with the normal "unixoid" representation too.
-     */
-    template<typename S>
-    S pathtoUNC(const S &str)
-    {
-        int len = 0;
-        S longStr;
-
-        len = str.length();
-        longStr.reserve(len+4);
-
-        // prepend \\?\ and convert '/' => '\' to support long names
-        if( str[0] == '/' || str[0] == '\\' ) {
-            // Don't prepend if already UNC
-            if( !(len > 1 && (str[1] == '/' || str[1] == '\\')) ) {
-                longStr.append(R"(\\?)");
-            }
-        } else {
-            longStr.append(R"(\\?\)"); // prepend string by this four magic chars.
-        }
-        longStr += str;
-
-        /* replace all occurences of / with the windows native \ */
-        
-        for (auto it = longStr.begin(); it != longStr.end(); ++it) {
-            if(*it == '/') {
-                *it = '\\';
-            }
-        }
-        return longStr;
-    }
 }
 
 /** @} */
