@@ -103,12 +103,6 @@ public:
     explicit SqlQuery() = default;
     explicit SqlQuery(SqlDatabase &db);
     explicit SqlQuery(const QByteArray &sql, SqlDatabase &db);
-    /**
-     * Prepare the SqlQuery if it was not prepared yet.
-     * Otherwise, clear the results and the bindings.
-     * return false if there is an error
-     */
-    bool initOrReset(const QByteArray &sql, SqlDatabase &db);
     /**
      * Prepare the SqlQuery.
      * If the query was already prepared, this will first call finish(), and re-prepare it.
@@ -161,10 +155,10 @@ public:
     const QByteArray &lastQuery() const;
     int numRowsAffected();
     void reset_and_clear_bindings();
-    void finish();
 
 private:
     void bindValueInternal(int pos, const QVariant &value);
+    void finish();
 
     SqlDatabase *_sqldb = nullptr;
     sqlite3 *_db = nullptr;
@@ -172,6 +166,9 @@ private:
     QString _error;
     int _errId;
     QByteArray _sql;
+
+    friend class SqlDatabase;
+    friend class PreparedSqlQueryManager;
 };
 
 } // namespace OCC
