@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include "common/utility.h"
+#include "common/filesystembase.h"
 #include "version.h"
 
 // Note:  This file must compile without QtGui
@@ -36,6 +37,7 @@
 #include <QStandardPaths>
 #include <QCollator>
 #include <QSysInfo>
+#include <qrandom.h>
 
 
 #ifdef Q_OS_UNIX
@@ -63,14 +65,13 @@ Q_LOGGING_CATEGORY(lcUtility, "nextcloud.sync.utility", QtInfoMsg)
 bool Utility::writeRandomFile(const QString &fname, int size)
 {
     int maxSize = 10 * 10 * 1024;
-    qsrand(QDateTime::currentMSecsSinceEpoch());
 
     if (size == -1)
-        size = qrand() % maxSize;
+        size = rand() % maxSize;
 
     QString randString;
     for (int i = 0; i < size; i++) {
-        int r = qrand() % 128;
+        int r = rand() % 128;
         randString.append(QChar(r));
     }
 
@@ -108,6 +109,11 @@ void Utility::setupFavLink(const QString &folder)
     setupFavLink_private(folder);
 }
 
+void Utility::removeFavLink(const QString &folder)
+{
+    removeFavLink_private(folder);
+}
+
 QString Utility::octetsToString(qint64 octets)
 {
 #define THE_FACTOR 1024
@@ -229,7 +235,7 @@ qint64 Utility::freeDiskSpace(const QString &path)
 #elif defined(Q_OS_WIN)
     ULARGE_INTEGER freeBytes;
     freeBytes.QuadPart = 0L;
-    if (GetDiskFreeSpaceEx(reinterpret_cast<const wchar_t *>(path.utf16()), &freeBytes, nullptr, nullptr)) {
+    if (GetDiskFreeSpaceEx(reinterpret_cast<const wchar_t *>(FileSystem::longWinPath(path).utf16()), &freeBytes, nullptr, nullptr)) {
         return freeBytes.QuadPart;
     }
 #endif
@@ -258,6 +264,11 @@ QString Utility::escape(const QString &in)
     return in.toHtmlEscaped();
 }
 
+int Utility::rand()
+{
+    return QRandomGenerator::global()->bounded(0, RAND_MAX);
+}
+
 void Utility::sleep(int sec)
 {
     QThread::sleep(sec);
