@@ -28,7 +28,7 @@ class AccountManager : public QObject
     Q_OBJECT
 public:
     static AccountManager *instance();
-    ~AccountManager() = default;
+    ~AccountManager() override = default;
 
     /**
      * Saves the accounts to a given settings file
@@ -58,7 +58,7 @@ public:
      * Return a list of all accounts.
      * (this is a list of QSharedPointer for internal reasons, one should normally not keep a copy of them)
      */
-    QList<AccountStatePtr> accounts() { return _accounts; }
+    QList<AccountStatePtr> accounts() const;
 
     /**
      * Return the account state pointer for an account identified by its display name
@@ -76,6 +76,12 @@ public:
      */
     static AccountPtr createAccount();
 
+    /**
+     * Returns the list of settings keys that can't be read because
+     * they are from the future.
+     */
+    static void backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys);
+
 private:
     // saving and loading Account to settings
     void saveAccountHelper(Account *account, QSettings &settings, bool saveCredentials = true);
@@ -91,6 +97,8 @@ private:
 
     AccountManager() = default;
     QList<AccountStatePtr> _accounts;
+    /// Account ids from settings that weren't read
+    QSet<QString> _additionalBlockedAccountIds;
 
 public slots:
     /// Saves account data, not including the credentials
@@ -106,6 +114,7 @@ public slots:
 Q_SIGNALS:
     void accountAdded(AccountState *account);
     void accountRemoved(AccountState *account);
+    void accountSyncConnectionRemoved(AccountState *account);
     void removeAccountFolders(AccountState *account);
 };
 }
