@@ -21,7 +21,7 @@
 #include <QPointer>
 #include "connectionvalidator.h"
 #include "creds/abstractcredentials.h"
-#include "userstatus.h"
+
 #include <memory>
 
 class QSettings;
@@ -82,7 +82,7 @@ public:
 
     /// Use the account as parent
     explicit AccountState(AccountPtr account);
-    ~AccountState();
+    ~AccountState() override;
 
     /** Creates an account state from settings and an Account object.
      *
@@ -162,19 +162,6 @@ public:
     ///Asks for user credentials
     void handleInvalidCredentials();
 
-    /** Returns the user status (Online, Dnd, Away, Offline, Invisible)
-     *  https://gist.github.com/georgehrke/55a0412007f13be1551d1f9436a39675
-    */
-    UserStatus::Status status() const;
-
-    /** Returns the user status Message (emoji + text)
-    */
-    QString statusMessage() const;
-
-    /** Returns the user status icon url
-    */
-    QUrl statusIcon() const;
-
     /** Returns the notifications status retrieved by the notificatons endpoint
      *  https://github.com/nextcloud/desktop/issues/2318#issuecomment-680698429
     */
@@ -184,9 +171,9 @@ public:
     */
     void setDesktopNotificationsAllowed(bool isAllowed);
 
-    /** Fetch the user status (status, icon, message)
-    */
-    void fetchUserStatus();
+    ConnectionStatus lastConnectionStatus() const;
+    
+    void trySignIn();
 
 public slots:
     /// Triggers a ping to the server to update state and
@@ -202,6 +189,7 @@ signals:
     void isConnectedChanged();
     void hasFetchedNavigationApps();
     void statusChanged();
+    void desktopNotificationsAllowedChanged();
 
 protected Q_SLOTS:
     void slotConnectionValidatorResult(ConnectionValidator::Status status, const QStringList &errors);
@@ -221,6 +209,7 @@ private:
     AccountPtr _account;
     State _state;
     ConnectionStatus _connectionStatus;
+    ConnectionStatus _lastConnectionValidatorStatus = ConnectionStatus::Undefined;
     QStringList _connectionErrors;
     bool _waitingForNewCredentials;
     QDateTime _timeOfLastETagCheck;
@@ -251,7 +240,6 @@ private:
      */
     AccountAppList _apps;
 
-    UserStatus *_userStatus;
     bool _isDesktopNotificationsAllowed;
 };
 
