@@ -42,7 +42,7 @@ public:
     };
 
     Flow2Auth(Account *account, QObject *parent);
-    ~Flow2Auth();
+    ~Flow2Auth() override;
 
     enum Result { NotSupported,
         LoggedIn,
@@ -81,6 +81,7 @@ private:
     qint64 _secondsInterval;
     bool _isBusy;
     bool _hasToken;
+    bool _enforceHttps = false;
 };
 
 } // namespace OCC
