@@ -48,7 +48,7 @@ void HttpCredentialsGui::askFromUserAsync()
     QObject::connect(job, &DetermineAuthTypeJob::authType, this, [this](DetermineAuthTypeJob::AuthType type) {
         if (type == DetermineAuthTypeJob::OAuth) {
             _asyncAuth.reset(new OAuth(_account, this));
-            _asyncAuth->_expectedUser = _user;
+            _asyncAuth->_expectedUser = _account->davUser();
             connect(_asyncAuth.data(), &OAuth::result,
                 this, &HttpCredentialsGui::asyncAuthResult);
             connect(_asyncAuth.data(), &OAuth::destroyed,
@@ -56,9 +56,7 @@ void HttpCredentialsGui::askFromUserAsync()
             _asyncAuth->start();
             emit authorisationLinkChanged();
         } else if (type == DetermineAuthTypeJob::Basic) {
-            // Show the dialog
-            // We will re-enter the event loop, so better wait the next iteration
-            QMetaObject::invokeMethod(this, "showDialog", Qt::QueuedConnection);
+            showDialog();
         } else {
             // Shibboleth?
             qCWarning(lcHttpCredentialsGui) << "Bad http auth type:" << type;
@@ -73,8 +71,7 @@ void HttpCredentialsGui::asyncAuthResult(OAuth::Result r, const QString &user,
 {
     switch (r) {
     case OAuth::NotSupported:
-        // We will re-enter the event loop, so better wait the next iteration
-        QMetaObject::invokeMethod(this, "showDialog", Qt::QueuedConnection);
+        showDialog();
         _asyncAuth.reset(nullptr);
         return;
     case OAuth::Error:
@@ -111,29 +108,32 @@ void HttpCredentialsGui::showDialog()
     }
     if (!_fetchErrorString.isEmpty()) {
         msg += QLatin1String("<br>")
-            + tr("Reading from keychain failed with error: '%1'")
+            + tr("Reading from keychain failed with error: \"%1\"")
                   .arg(Utility::escape(_fetchErrorString))
             + QLatin1String("<br>");
     }
 
-    QInputDialog dialog;
-    dialog.setWindowTitle(tr("Enter Password"));
-    dialog.setLabelText(msg);
-    dialog.setTextValue(_previousPassword);
-    dialog.setTextEchoMode(QLineEdit::Password);
-    if (auto *dialogLabel = dialog.findChild<QLabel *>()) {
+    auto *dialog = new QInputDialog();
+    dialog->setAttribute(Qt::WA_DeleteOnClose, true);
+    dialog->setWindowTitle(tr("Enter Password"));
+    dialog->setLabelText(msg);
+    dialog->setTextValue(_previousPassword);
+    dialog->setTextEchoMode(QLineEdit::Password);
+    if (auto *dialogLabel = dialog->findChild<QLabel *>()) {
         dialogLabel->setOpenExternalLinks(true);
         dialogLabel->setTextFormat(Qt::RichText);
     }
 
-    bool ok = dialog.exec();
-    if (ok) {
-        _password = dialog.textValue();
-        _refreshToken.clear();
-        _ready = true;
-        persist();
-    }
-    emit asked();
+    dialog->open();
+    connect(dialog, &QDialog::finished, this, [this, dialog](int result) {
+        if (result == QDialog::Accepted) {
+            _password = dialog->textValue();
+            _refreshToken.clear();
+            _ready = true;
+            persist();
+        }
+        emit asked();
+    });
 }
 
 QString HttpCredentialsGui::requestAppPasswordText(const Account *account)
