@@ -21,7 +21,7 @@
 #include "configfile.h"
 #include "owncloudsetupwizard.h"
 #include "accountmanager.h"
-#include "synclogdialog.h"
+#include "guiutility.h"
 
 #if defined(BUILD_UPDATER)
 #include "updater/updater.h"
@@ -157,8 +157,17 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     _ui->showInExplorerNavigationPaneCheckBox->setText(txt);
 #endif
 
-    _ui->autostartCheckBox->setChecked(Utility::hasLaunchOnStartup(Theme::instance()->appName()));
-    connect(_ui->autostartCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleLaunchOnStartup);
+    if(Utility::hasSystemLaunchOnStartup(Theme::instance()->appName())) {
+        _ui->autostartCheckBox->setChecked(true);
+        _ui->autostartCheckBox->setDisabled(true);
+        _ui->autostartCheckBox->setToolTip(tr("You cannot disable autostart because system-wide autostart is enabled."));
+    } else {
+        const bool hasAutoStart = Utility::hasLaunchOnStartup(Theme::instance()->appName());
+        // make sure the binary location is correctly set
+        slotToggleLaunchOnStartup(hasAutoStart);
+        _ui->autostartCheckBox->setChecked(hasAutoStart);
+        connect(_ui->autostartCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleLaunchOnStartup);
+    }
 
     // setup about section
     QString about = Theme::instance()->about();
@@ -224,7 +233,6 @@ GeneralSettings::GeneralSettings(QWidget *parent)
 GeneralSettings::~GeneralSettings()
 {
     delete _ui;
-    delete _syncLogDialog;
 }
 
 QSize GeneralSettings::sizeHint() const
@@ -268,8 +276,13 @@ void GeneralSettings::slotUpdateInfo()
         connect(_ui->updateButton, &QAbstractButton::clicked, this, &GeneralSettings::slotUpdateCheckNow, Qt::UniqueConnection);
         connect(_ui->autoCheckForUpdatesCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleAutoUpdateCheck);
 
-        QString status = ocupdater->statusString();
+        QString status = ocupdater->statusString(OCUpdater::UpdateStatusStringFormat::Html);
         Theme::replaceLinkColorStringBackgroundAware(status);
+
+        _ui->updateStateLabel->setOpenExternalLinks(false);
+        connect(_ui->updateStateLabel, &QLabel::linkActivated, this, [](const QString &link) {
+            Utility::openBrowser(QUrl(link));
+        });
         _ui->updateStateLabel->setText(status);
 
         _ui->restartButton->setVisible(ocupdater->downloadState() == OCUpdater::DownloadComplete);
@@ -314,11 +327,11 @@ void GeneralSettings::slotUpdateChannelChanged(const QString &channel)
            "version."),
         QMessageBox::NoButton,
         this);
-    msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
+    auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
     msgBox->addButton(tr("Cancel"), QMessageBox::RejectRole);
-    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox](int result) {
+    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton] {
         msgBox->deleteLater();
-        if (result == QMessageBox::AcceptRole) {
+        if (msgBox->clickedButton() == acceptButton) {
             ConfigFile().setUpdateChannel(channel);
             if (auto updater = qobject_cast<OCUpdater *>(Updater::instance())) {
                 updater->setUpdateUrl(Updater::updateUrl());
