@@ -91,7 +91,7 @@ void IgnoreListTableWidget::slotWriteIgnoreFile(const QString & file)
         }
     } else {
         QMessageBox::warning(this, tr("Could not open file"),
-            tr("Cannot write changes to '%1'.").arg(file));
+            tr("Cannot write changes to \"%1\".").arg(file));
     }
     ignores.close(); //close the file before reloading stuff.
 
