@@ -38,7 +38,7 @@ public:
      *
      * @param path Path to request shares for (default all shares)
      */
-    void getSharees(const QString &search, const QString &itemType, int page = 1, int perPage = 50);
+    void getSharees(const QString &search, const QString &itemType, int page = 1, int perPage = 50, bool lookup = false);
 signals:
     /**
      * Result of the OCS request
