@@ -12,6 +12,9 @@
  * for more details.
  */
 
+#include "ocsprofileconnector.h"
+#include "sharee.h"
+#include "tray/usermodel.h"
 #include "ui_shareusergroupwidget.h"
 #include "ui_shareuserline.h"
 #include "shareusergroupwidget.h"
@@ -26,6 +29,7 @@
 #include "thumbnailjob.h"
 #include "sharemanager.h"
 #include "theme.h"
+#include "iconutils.h"
 
 #include "QProgressIndicator.h"
 #include <QBuffer>
@@ -34,21 +38,51 @@
 #include <QFileInfo>
 #include <QAbstractProxyModel>
 #include <QCompleter>
-#include <qlayout.h>
+#include <QBoxLayout>
+#include <QIcon>
+#include <QLayout>
 #include <QPropertyAnimation>
 #include <QMenu>
 #include <QAction>
 #include <QDesktopServices>
+#include <QInputDialog>
 #include <QMessageBox>
 #include <QCryptographicHash>
 #include <QColor>
 #include <QPainter>
 #include <QListWidget>
+#include <QSvgRenderer>
+#include <QPushButton>
+#include <QContextMenuEvent>
 
 #include <cstring>
 
+namespace {
+const char *passwordIsSetPlaceholder = "●●●●●●●●";
+
+}
+
 namespace OCC {
 
+AvatarEventFilter::AvatarEventFilter(QObject *parent)
+    : QObject(parent)
+{
+}
+
+
+bool AvatarEventFilter::eventFilter(QObject *obj, QEvent *event)
+{
+    if (event->type() == QEvent::ContextMenu) {
+        const auto contextMenuEvent = dynamic_cast<QContextMenuEvent *>(event);
+        if (!contextMenuEvent) {
+            return false;
+        }
+        emit contextMenu(contextMenuEvent->globalPos());
+        return true;
+    }
+    return QObject::eventFilter(obj, event);
+}
+
 ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     const QString &sharePath,
     const QString &localPath,
@@ -96,7 +130,7 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
 
     _manager = new ShareManager(_account, this);
     connect(_manager, &ShareManager::sharesFetched, this, &ShareUserGroupWidget::slotSharesFetched);
-    connect(_manager, &ShareManager::shareCreated, this, &ShareUserGroupWidget::getShares);
+    connect(_manager, &ShareManager::shareCreated, this, &ShareUserGroupWidget::slotShareCreated);
     connect(_manager, &ShareManager::serverError, this, &ShareUserGroupWidget::displayError);
     connect(_ui->shareeLineEdit, &QLineEdit::returnPressed, this, &ShareUserGroupWidget::slotLineEditReturn);
     connect(_ui->confirmShare, &QAbstractButton::clicked, this, &ShareUserGroupWidget::slotLineEditReturn);
@@ -201,6 +235,16 @@ void ShareUserGroupWidget::getShares()
     _manager->fetchShares(_sharePath);
 }
 
+void ShareUserGroupWidget::slotShareCreated(const QSharedPointer<Share> &share)
+{
+    if (share && _account->capabilities().shareEmailPasswordEnabled() && !_account->capabilities().shareEmailPasswordEnforced()) {
+        // remember this share Id so we can set it's password Line Edit to focus later
+        _lastCreatedShareId = share->getId();
+    }
+    // fetch all shares including the one we've just created
+    getShares();
+}
+
 void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shares)
 {
     QScrollArea *scrollArea = _parentScrollArea;
@@ -212,6 +256,8 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
     int height = 0;
     QList<QString> linkOwners({});
 
+    ShareUserLine *justCreatedShareThatNeedsPassword = nullptr;
+
     foreach (const auto &share, shares) {
         // We don't handle link shares, only TypeUser or TypeGroup
         if (share->getShareType() == Share::TypeLink) {
@@ -228,7 +274,10 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
             _ui->mainOwnerLabel->setText(QString("Shared with you by ").append(share->getOwnerDisplayName()));
         }
 
-        auto *s = new ShareUserLine(share, _maxSharingPermissions, _isFile, _parentScrollArea);
+
+        Q_ASSERT(Share::isShareTypeUserGroupEmailRoomOrRemote(share->getShareType()));
+        auto userGroupShare = qSharedPointerDynamicCast<UserGroupShare>(share);
+        auto *s = new ShareUserLine(_account, userGroupShare, _maxSharingPermissions, _isFile, _parentScrollArea);
         connect(s, &ShareUserLine::resizeRequested, this, &ShareUserGroupWidget::slotAdjustScrollWidgetSize);
         connect(s, &ShareUserLine::visualDeletionDone, this, &ShareUserGroupWidget::getShares);
         s->setBackgroundRole(layout->count() % 2 == 0 ? QPalette::Base : QPalette::AlternateBase);
@@ -238,6 +287,13 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
 
         layout->addWidget(s);
 
+        if (!_lastCreatedShareId.isEmpty() && share->getId() == _lastCreatedShareId) {
+            _lastCreatedShareId = QString();
+            if (_account->capabilities().shareEmailPasswordEnabled() && !_account->capabilities().shareEmailPasswordEnforced()) {
+                justCreatedShareThatNeedsPassword = s;
+            }
+        }
+
         x++;
         if (x <= 3) {
             height = newViewPort->sizeHint().height();
@@ -262,17 +318,29 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
 
     _disableCompleterActivated = false;
     activateShareeLineEdit();
+
+    if (justCreatedShareThatNeedsPassword) {
+        // always set focus to a password Line Edit when the new email share is created on a server with optional passwords enabled for email shares
+        justCreatedShareThatNeedsPassword->focusPasswordLineEdit();
+    }
 }
 
 void ShareUserGroupWidget::slotAdjustScrollWidgetSize()
 {
     QScrollArea *scrollArea = _parentScrollArea;
-    int count = scrollArea->findChildren<ShareUserLine *>().count();
-    scrollArea->setVisible(count > 0);
-    if (count > 0 && count <= 3) {
+    const auto shareUserLineChilds = scrollArea->findChildren<ShareUserLine *>();
+
+    // Ask the child widgets to calculate their size
+    for (const auto shareUserLineChild : shareUserLineChilds) {
+        shareUserLineChild->adjustSize();
+    }
+
+    const auto shareUserLineChildsCount = shareUserLineChilds.count();
+    scrollArea->setVisible(shareUserLineChildsCount > 0);
+    if (shareUserLineChildsCount > 0 && shareUserLineChildsCount <= 3) {
         scrollArea->setFixedHeight(scrollArea->widget()->sizeHint().height());
     }
-    scrollArea->setFrameShape(count > 3 ? QFrame::StyledPanel : QFrame::NoFrame);
+    scrollArea->setFrameShape(shareUserLineChildsCount > 3 ? QFrame::StyledPanel : QFrame::NoFrame);
 }
 
 void ShareUserGroupWidget::slotPrivateLinkShare()
@@ -294,7 +362,7 @@ void ShareUserGroupWidget::slotShareesReady()
 
     _pi_sharee.stopAnimation();
     if (_completerModel->rowCount() == 0) {
-        displayError(0, tr("No results for '%1'").arg(_completerModel->currentSearch()));
+        displayError(0, tr("No results for \"%1\"").arg(_completerModel->currentSearch()));
     }
 
     // if no rows are present in the model - complete() will hide the completer
@@ -327,25 +395,35 @@ void ShareUserGroupWidget::slotCompleterActivated(const QModelIndex &index)
      * https://github.com/owncloud/core/issues/22122#issuecomment-185637344
      * https://github.com/owncloud/client/issues/4996
      */
-    if (sharee->type() == Sharee::Federated
-        && _account->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        int permissions = SharePermissionRead | SharePermissionUpdate;
-        if (!_isFile) {
-            permissions |= SharePermissionCreate | SharePermissionDelete;
-        }
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), SharePermission(permissions));
-    } else {
 
-        // Default permissions on creation
-        int permissions = SharePermissionCreate | SharePermissionUpdate
-                | SharePermissionDelete | SharePermissionShare;
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), SharePermission(permissions));
+    _lastCreatedShareId = QString();
+    
+    QString password;
+    if (sharee->type() == Sharee::Email && _account->capabilities().shareEmailPasswordEnforced()) {
+        _ui->shareeLineEdit->clear();
+        // always show a dialog for password-enforced email shares
+        bool ok = false;
+        
+        do {
+            password = QInputDialog::getText(
+                this,
+                tr("Password for share required"),
+                tr("Please enter a password for your email share:"),
+                QLineEdit::Password,
+                QString(),
+                &ok);
+        } while (password.isEmpty() && ok);
+        
+        if (!ok) {
+            return;
+        }
     }
+    
+    _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
+        sharee->shareWith(), _maxSharingPermissions, password);
 
     _ui->shareeLineEdit->setEnabled(false);
-    _ui->shareeLineEdit->setText(QString());
+    _ui->shareeLineEdit->clear();
 }
 
 void ShareUserGroupWidget::slotCompleterHighlighted(const QModelIndex &index)
@@ -413,15 +491,16 @@ void ShareUserGroupWidget::activateShareeLineEdit()
     _ui->shareeLineEdit->setFocus();
 }
 
-ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
-    SharePermissions maxSharingPermissions,
-    bool isFile,
-    QWidget *parent)
+ShareUserLine::ShareUserLine(AccountPtr account, QSharedPointer<UserGroupShare> share,
+    SharePermissions maxSharingPermissions, bool isFile, QWidget *parent)
     : QWidget(parent)
     , _ui(new Ui::ShareUserLine)
+    , _account(account)
     , _share(share)
     , _isFile(isFile)
+    , _profilePageMenu(account, share->getShareWith()->shareWith())
 {
+    Q_ASSERT(_share);
     _ui->setupUi(this);
 
     _ui->sharedWith->setElideMode(Qt::ElideRight);
@@ -434,6 +513,15 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
                                       maxSharingPermissions & SharePermissionDelete);
     _ui->permissionsEdit->setEnabled(enabled);
     connect(_ui->permissionsEdit, &QAbstractButton::clicked, this, &ShareUserLine::slotEditPermissionsChanged);
+    connect(_ui->noteConfirmButton, &QAbstractButton::clicked, this, &ShareUserLine::onNoteConfirmButtonClicked);
+    connect(_ui->calendar, &QDateTimeEdit::dateChanged, this, &ShareUserLine::setExpireDate);
+
+    connect(_share.data(), &UserGroupShare::noteSet, this, &ShareUserLine::disableProgessIndicatorAnimation);
+    connect(_share.data(), &UserGroupShare::noteSetError, this, &ShareUserLine::disableProgessIndicatorAnimation);
+    connect(_share.data(), &UserGroupShare::expireDateSet, this, &ShareUserLine::disableProgessIndicatorAnimation);
+
+    connect(_ui->confirmPassword, &QToolButton::clicked, this, &ShareUserLine::slotConfirmPasswordClicked);
+    connect(_ui->lineEdit_password, &QLineEdit::returnPressed, this, &ShareUserLine::slotLineEditPasswordReturnPressed);
 
     // create menu with checkable permissions
     auto *menu = new QMenu(this);
@@ -443,6 +531,38 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
     menu->addAction(_permissionReshare);
     connect(_permissionReshare, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
 
+    showNoteOptions(false);
+
+    const bool isNoteSupported = _share->getShareType() != Share::ShareType::TypeEmail && _share->getShareType() != Share::ShareType::TypeRoom;
+
+    if (isNoteSupported) {
+        _noteLinkAction = new QAction(tr("Note to recipient"));
+        _noteLinkAction->setCheckable(true);
+        menu->addAction(_noteLinkAction);
+        connect(_noteLinkAction, &QAction::triggered, this, &ShareUserLine::toggleNoteOptions);
+        if (!_share->getNote().isEmpty()) {
+            _noteLinkAction->setChecked(true);
+            showNoteOptions(true);
+        }
+    }
+
+    showExpireDateOptions(false);
+
+    const bool isExpirationDateSupported = _share->getShareType() != Share::ShareType::TypeEmail;
+
+    if (isExpirationDateSupported) {
+        // email shares do not support expiration dates
+        _expirationDateLinkAction = new QAction(tr("Set expiration date"));
+        _expirationDateLinkAction->setCheckable(true);
+        menu->addAction(_expirationDateLinkAction);
+        connect(_expirationDateLinkAction, &QAction::triggered, this, &ShareUserLine::toggleExpireDateOptions);
+        const auto expireDate = _share->getExpireDate().isValid() ? share.data()->getExpireDate() : QDate();
+        if (!expireDate.isNull()) {
+            _expirationDateLinkAction->setChecked(true);
+            showExpireDateOptions(true, expireDate);
+        }
+    }
+
     menu->addSeparator();
 
       // Adds action to delete share widget
@@ -475,9 +595,32 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
         connect(_permissionDelete, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
     }
 
+    // Adds action to display password widget (check box)
+    if (_share->getShareType() == Share::TypeEmail && (_share->isPasswordSet() || _account->capabilities().shareEmailPasswordEnabled())) {
+        _passwordProtectLinkAction = new QAction(tr("Password protect"), this);
+        _passwordProtectLinkAction->setCheckable(true);
+        _passwordProtectLinkAction->setChecked(_share->isPasswordSet());
+        // checkbox can be checked/unchedkec if the password is not yet set or if it's not enforced
+        _passwordProtectLinkAction->setEnabled(!_share->isPasswordSet() || !_account->capabilities().shareEmailPasswordEnforced());
+
+        menu->addAction(_passwordProtectLinkAction);
+        connect(_passwordProtectLinkAction, &QAction::triggered, this, &ShareUserLine::slotPasswordCheckboxChanged);
+
+        refreshPasswordLineEditPlaceholder();
+
+        connect(_share.data(), &Share::passwordSet, this, &ShareUserLine::slotPasswordSet);
+        connect(_share.data(), &Share::passwordSetError, this, &ShareUserLine::slotPasswordSetError);
+    }
+
+    refreshPasswordOptions();
+
+    _ui->errorLabel->hide();
+
     _ui->permissionToolButton->setMenu(menu);
     _ui->permissionToolButton->setPopupMode(QToolButton::InstantPopup);
 
+    _ui->passwordProgressIndicator->setVisible(false);
+
     // Set the permissions checkboxes
     displayPermissions();
 
@@ -499,11 +642,22 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
         _permissionReshare->setVisible(false);
     }
 
+    const auto avatarEventFilter = new AvatarEventFilter(_ui->avatar);
+    connect(avatarEventFilter, &AvatarEventFilter::contextMenu, this, &ShareUserLine::onAvatarContextMenu);
+    _ui->avatar->installEventFilter(avatarEventFilter);
+
     loadAvatar();
 
     customizeStyle();
 }
 
+void ShareUserLine::onAvatarContextMenu(const QPoint &globalPosition)
+{
+    if (_share->getShareType() == Share::TypeUser) {
+        _profilePageMenu.exec(globalPosition);
+    }
+}
+
 void ShareUserLine::loadAvatar()
 {
     const int avatarSize = 36;
@@ -515,28 +669,7 @@ void ShareUserLine::loadAvatar()
     _ui->avatar->setMaximumWidth(avatarSize);
     _ui->avatar->setAlignment(Qt::AlignCenter);
 
-    /* Create the fallback avatar.
-     *
-     * This will be shown until the avatar image data arrives.
-     */
-    const QByteArray hash = QCryptographicHash::hash(_ui->sharedWith->text().toUtf8(), QCryptographicHash::Md5);
-    double hue = static_cast<quint8>(hash[0]) / 255.;
-
-    // See core/js/placeholder.js for details on colors and styling
-    const QColor bg = QColor::fromHslF(hue, 0.7, 0.68);
-    const QString style = QString(R"(* {
-        color: #fff;
-        background-color: %1;
-        border-radius: %2px;
-        text-align: center;
-        line-height: %2px;
-        font-size: %2px;
-    })").arg(bg.name(), QString::number(avatarSize / 2));
-    _ui->avatar->setStyleSheet(style);
-
-    // The avatar label is the first character of the user name.
-    const QString text = _share->getShareWith()->displayName();
-    _ui->avatar->setText(text.at(0).toUpper());
+    setDefaultAvatar(avatarSize);
 
     /* Start the network job to fetch the avatar data.
      *
@@ -549,6 +682,38 @@ void ShareUserLine::loadAvatar()
     }
 }
 
+void ShareUserLine::setDefaultAvatar(int avatarSize)
+{
+    /* Create the fallback avatar.
+     *
+     * This will be shown until the avatar image data arrives.
+     */
+
+    // See core/js/placeholder.js for details on colors and styling
+    const auto backgroundColor = backgroundColorForShareeType(_share->getShareWith()->type());
+    const QString style = QString(R"(* {
+        color: #fff;
+        background-color: %1;
+        border-radius: %2px;
+        text-align: center;
+        line-height: %2px;
+        font-size: %2px;
+    })").arg(backgroundColor.name(), QString::number(avatarSize / 2));
+    _ui->avatar->setStyleSheet(style);
+
+    const auto pixmap = pixmapForShareeType(_share->getShareWith()->type(), backgroundColor);
+
+    if (!pixmap.isNull()) {
+        _ui->avatar->setPixmap(pixmap);
+    } else {
+        qCDebug(lcSharing) << "pixmap is null for share type: " << _share->getShareWith()->type();
+
+        // The avatar label is the first character of the user name.
+        const auto text = _share->getShareWith()->displayName();
+        _ui->avatar->setText(text.at(0).toUpper());
+    }
+}
+
 void ShareUserLine::slotAvatarLoaded(QImage avatar)
 {
     if (avatar.isNull())
@@ -634,6 +799,29 @@ void ShareUserLine::slotPermissionsChanged()
     _share->setPermissions(permissions);
 }
 
+void ShareUserLine::slotPasswordCheckboxChanged()
+{
+    if (!_passwordProtectLinkAction->isChecked()) {
+        _ui->errorLabel->hide();
+        _ui->errorLabel->clear();
+
+        if (!_share->isPasswordSet()) {
+            _ui->lineEdit_password->clear();
+            refreshPasswordOptions();
+        } else {
+            // do not call refreshPasswordOptions here, as it will be called after the network request is complete
+            togglePasswordSetProgressAnimation(true);
+            _share->setPassword(QString());
+        }
+    } else {
+        refreshPasswordOptions();
+
+        if (_ui->lineEdit_password->isVisible() && _ui->lineEdit_password->isEnabled()) {
+            focusPasswordLineEdit();
+        }
+    }
+}
+
 void ShareUserLine::slotDeleteAnimationFinished()
 {
     emit resizeRequested();
@@ -646,6 +834,63 @@ void ShareUserLine::slotDeleteAnimationFinished()
     connect(this, SIGNAL(destroyed(QObject *)), parentWidget(), SLOT(repaint()));
 }
 
+void ShareUserLine::refreshPasswordOptions()
+{
+    const bool isPasswordEnabled = _share->getShareType() == Share::TypeEmail && _passwordProtectLinkAction->isChecked();
+
+    _ui->passwordLabel->setVisible(isPasswordEnabled);
+    _ui->lineEdit_password->setEnabled(isPasswordEnabled);
+    _ui->lineEdit_password->setVisible(isPasswordEnabled);
+    _ui->confirmPassword->setVisible(isPasswordEnabled);
+
+    emit resizeRequested();
+}
+
+void ShareUserLine::refreshPasswordLineEditPlaceholder()
+{
+    if (_share->isPasswordSet()) {
+        _ui->lineEdit_password->setPlaceholderText(QString::fromUtf8(passwordIsSetPlaceholder));
+    } else {
+        _ui->lineEdit_password->setPlaceholderText("");
+    }
+}
+
+void ShareUserLine::slotPasswordSet()
+{
+    togglePasswordSetProgressAnimation(false);
+    _ui->lineEdit_password->setEnabled(true);
+    _ui->confirmPassword->setEnabled(true);
+
+    _ui->lineEdit_password->setText("");
+
+    _passwordProtectLinkAction->setEnabled(!_share->isPasswordSet() || !_account->capabilities().shareEmailPasswordEnforced());
+
+    refreshPasswordLineEditPlaceholder();
+
+    refreshPasswordOptions();
+}
+
+void ShareUserLine::slotPasswordSetError(int statusCode, const QString &message)
+{
+    qCWarning(lcSharing) << "Error from server" << statusCode << message;
+
+    togglePasswordSetProgressAnimation(false);
+
+    _ui->lineEdit_password->setEnabled(true);
+    _ui->confirmPassword->setEnabled(true);
+
+    refreshPasswordLineEditPlaceholder();
+
+    refreshPasswordOptions();
+
+    focusPasswordLineEdit();
+
+    _ui->errorLabel->show();
+    _ui->errorLabel->setText(message);
+
+    emit resizeRequested();
+}
+
 void ShareUserLine::slotShareDeleted()
 {
     auto *animation = new QPropertyAnimation(this, "maximumHeight", this);
@@ -702,12 +947,224 @@ void ShareUserLine::slotStyleChanged()
     customizeStyle();
 }
 
+void ShareUserLine::focusPasswordLineEdit()
+{
+    _ui->lineEdit_password->setFocus();
+}
+
 void ShareUserLine::customizeStyle()
 {
     _ui->permissionToolButton->setIcon(Theme::createColorAwareIcon(":/client/theme/more.svg"));
 
     QIcon deleteicon = QIcon::fromTheme(QLatin1String("user-trash"),Theme::createColorAwareIcon(QLatin1String(":/client/theme/delete.svg")));
     _deleteShareButton->setIcon(deleteicon);
+
+    _ui->noteConfirmButton->setIcon(Theme::createColorAwareIcon(":/client/theme/confirm.svg"));
+    _ui->progressIndicator->setColor(QGuiApplication::palette().color(QPalette::WindowText));
+
+    // make sure to force BackgroundRole to QPalette::WindowText for a lable, because it's parent always has a different role set that applies to children unless customized
+    _ui->errorLabel->setBackgroundRole(QPalette::WindowText);
 }
 
+QPixmap ShareUserLine::pixmapForShareeType(Sharee::Type type, const QColor &backgroundColor) const
+{
+    switch (type) {
+    case Sharee::Room:
+        return Ui::IconUtils::pixmapForBackground(QStringLiteral("talk-app.svg"), backgroundColor);
+    case Sharee::Email:
+        return Ui::IconUtils::pixmapForBackground(QStringLiteral("email.svg"), backgroundColor);
+    case Sharee::Group:
+    case Sharee::Federated:
+    case Sharee::Circle:
+    case Sharee::User:
+        break;
+    }
+
+    return {};
+}
+
+QColor ShareUserLine::backgroundColorForShareeType(Sharee::Type type) const
+{
+    switch (type) {
+    case Sharee::Room:
+        return Theme::instance()->wizardHeaderBackgroundColor();
+    case Sharee::Email:
+        return Theme::instance()->wizardHeaderTitleColor();
+    case Sharee::Group:
+    case Sharee::Federated:
+    case Sharee::Circle:
+    case Sharee::User:
+        break;
+    }
+
+    const auto calculateBackgroundBasedOnText = [this]() {
+        const auto hash = QCryptographicHash::hash(_ui->sharedWith->text().toUtf8(), QCryptographicHash::Md5);
+        Q_ASSERT(hash.size() > 0);
+        if (hash.size() == 0) {
+            qCWarning(lcSharing) << "Failed to calculate hash color for share:" << _share->path();
+            return QColor{};
+        }
+        const double hue = static_cast<quint8>(hash[0]) / 255.;
+        return QColor::fromHslF(hue, 0.7, 0.68);
+    };
+
+    return calculateBackgroundBasedOnText();
+}
+
+void ShareUserLine::showNoteOptions(bool show)
+{
+    _ui->noteLabel->setVisible(show);
+    _ui->noteTextEdit->setVisible(show);
+    _ui->noteConfirmButton->setVisible(show);
+
+    if (show) {
+        const auto note = _share->getNote();
+        _ui->noteTextEdit->setText(note);
+        _ui->noteTextEdit->setFocus();
+    }
+
+    emit resizeRequested();
+}
+
+
+void ShareUserLine::toggleNoteOptions(bool enable)
+{
+    showNoteOptions(enable);
+
+    if (!enable) {
+        // Delete note
+        _share->setNote(QString());
+    }
+}
+
+void ShareUserLine::onNoteConfirmButtonClicked()
+{
+    setNote(_ui->noteTextEdit->toPlainText());
+}
+
+void ShareUserLine::setNote(const QString &note)
+{
+    enableProgessIndicatorAnimation(true);
+    _share->setNote(note);
+}
+
+void ShareUserLine::toggleExpireDateOptions(bool enable)
+{
+    showExpireDateOptions(enable);
+
+    if (!enable) {
+        _share->setExpireDate(QDate());
+    }
+}
+
+void ShareUserLine::showExpireDateOptions(bool show, const QDate &initialDate)
+{
+    _ui->expirationLabel->setVisible(show);
+    _ui->calendar->setVisible(show);
+
+    if (show) {
+        _ui->calendar->setMinimumDate(QDate::currentDate().addDays(1));
+        _ui->calendar->setDate(initialDate.isValid() ? initialDate : _ui->calendar->minimumDate());
+        _ui->calendar->setFocus();
+
+        if (enforceExpirationDateForShare(_share->getShareType())) {
+            _ui->calendar->setMaximumDate(maxExpirationDateForShare(_share->getShareType(), _ui->calendar->maximumDate()));
+            _expirationDateLinkAction->setChecked(true);
+            _expirationDateLinkAction->setEnabled(false);
+        }
+    }
+
+    emit resizeRequested();
+}
+
+void ShareUserLine::setExpireDate()
+{
+    enableProgessIndicatorAnimation(true);
+    _share->setExpireDate(_ui->calendar->date());
+}
+
+void ShareUserLine::enableProgessIndicatorAnimation(bool enable)
+{
+    if (enable) {
+        if (!_ui->progressIndicator->isAnimated()) {
+            _ui->progressIndicator->startAnimation();
+        }
+    } else {
+        _ui->progressIndicator->stopAnimation();
+    }
+}
+
+void ShareUserLine::togglePasswordSetProgressAnimation(bool show)
+{
+    // button and progress indicator are interchanged depending on if the network request is in progress or not
+    _ui->confirmPassword->setVisible(!show && _passwordProtectLinkAction->isChecked());
+    _ui->passwordProgressIndicator->setVisible(show);
+    if (show) {
+        if (!_ui->passwordProgressIndicator->isAnimated()) {
+            _ui->passwordProgressIndicator->startAnimation();
+        }
+    } else {
+        _ui->passwordProgressIndicator->stopAnimation();
+    }
+}
+
+void ShareUserLine::disableProgessIndicatorAnimation()
+{
+    enableProgessIndicatorAnimation(false);
+}
+
+QDate ShareUserLine::maxExpirationDateForShare(const Share::ShareType type, const QDate &fallbackDate) const
+{
+    auto daysToExpire = 0;
+    if (type == Share::ShareType::TypeRemote) {
+        daysToExpire = _account->capabilities().shareRemoteExpireDateDays();
+    } else if (type == Share::ShareType::TypeEmail) {
+       daysToExpire = _account->capabilities().sharePublicLinkExpireDateDays();
+    } else {
+        daysToExpire = _account->capabilities().shareInternalExpireDateDays();
+    }
+
+    if (daysToExpire > 0) {
+        return QDate::currentDate().addDays(daysToExpire);
+    }
+
+    return fallbackDate;
+}
+
+bool ShareUserLine::enforceExpirationDateForShare(const Share::ShareType type) const
+{
+    if (type == Share::ShareType::TypeRemote) {
+        return _account->capabilities().shareRemoteEnforceExpireDate();
+    } else if (type == Share::ShareType::TypeEmail) {
+        return _account->capabilities().sharePublicLinkEnforceExpireDate();
+    }
+
+    return _account->capabilities().shareInternalEnforceExpireDate();
+}
+
+void ShareUserLine::setPasswordConfirmed()
+{
+    if (_ui->lineEdit_password->text().isEmpty()) {
+        return;
+    }
+
+    _ui->lineEdit_password->setEnabled(false);
+    _ui->confirmPassword->setEnabled(false);
+
+    _ui->errorLabel->hide();
+    _ui->errorLabel->clear();
+
+    togglePasswordSetProgressAnimation(true);
+    _share->setPassword(_ui->lineEdit_password->text());
+}
+
+void ShareUserLine::slotLineEditPasswordReturnPressed()
+{
+    setPasswordConfirmed();
+}
+
+void ShareUserLine::slotConfirmPasswordClicked()
+{
+    setPasswordConfirmed();
+}
 }
