@@ -17,7 +17,12 @@
 #include "theme.h"
 #include "config.h"
 #include "common/utility.h"
-#include "tray/UserModel.h"
+#include "tray/svgimageprovider.h"
+#include "tray/usermodel.h"
+#include "wheelhandler.h"
+#include "tray/unifiedsearchresultimageprovider.h"
+#include "configfile.h"
+#include "accessmanager.h"
 
 #include <QCursor>
 #include <QGuiApplication>
@@ -55,8 +60,12 @@ void Systray::setTrayEngine(QQmlApplicationEngine *trayEngine)
 {
     _trayEngine = trayEngine;
 
+    _trayEngine->setNetworkAccessManagerFactory(&_accessManagerFactory);
+
     _trayEngine->addImportPath("qrc:/qml/theme");
     _trayEngine->addImageProvider("avatars", new ImageProvider);
+    _trayEngine->addImageProvider(QLatin1String("svgimage-custom-color"), new OCC::Ui::SvgImageProvider);
+    _trayEngine->addImageProvider(QLatin1String("unified-search-result-icon"), new UnifiedSearchResultImageProvider);
 }
 
 Systray::Systray()
@@ -86,6 +95,8 @@ Systray::Systray()
         }
     );
 
+    qmlRegisterType<WheelHandler>("com.nextcloud.desktopclient", 1, 0, "WheelHandler");
+
 #ifndef Q_OS_MAC
     auto contextMenu = new QMenu();
     if (AccountManager::instance()->accounts().isEmpty()) {
@@ -197,6 +208,21 @@ bool Systray::isOpen()
     return _isOpen;
 }
 
+QString Systray::windowTitle() const
+{
+    return Theme::instance()->appNameGUI();
+}
+
+bool Systray::useNormalWindow() const
+{
+    if (!isSystemTrayAvailable()) {
+        return true;
+    }
+
+    ConfigFile cfg;
+    return cfg.showMainDialogAsNormalWindow();
+}
+
 Q_INVOKABLE void Systray::setOpened()
 {
     _isOpen = true;
@@ -256,10 +282,11 @@ void Systray::pauseResumeSync()
 
 void Systray::positionWindow(QQuickWindow *window) const
 {
-    window->setScreen(currentScreen());
-
-    const auto position = computeWindowPosition(window->width(), window->height());
-    window->setPosition(position);
+    if (!useNormalWindow()) {
+        window->setScreen(currentScreen());
+        const auto position = computeWindowPosition(window->width(), window->height());
+        window->setPosition(position);
+    }
 }
 
 void Systray::forceWindowInit(QQuickWindow *window) const
@@ -481,4 +508,14 @@ QPoint Systray::calcTrayIconCenter() const
 #endif
 }
 
+AccessManagerFactory::AccessManagerFactory()
+    : QQmlNetworkAccessManagerFactory()
+{
+}
+
+QNetworkAccessManager* AccessManagerFactory::create(QObject *parent)
+{
+    return new AccessManager(parent);
+}
+
 } // namespace OCC
