@@ -16,6 +16,10 @@
 #include <QLabel>
 #include <QPixmap>
 #include <QVariant>
+#include <QRadioButton>
+#include <QAbstractButton>
+#include <QCheckBox>
+#include <QSpinBox>
 
 #include "wizard/owncloudwizardcommon.h"
 #include "theme.h"
@@ -68,6 +72,15 @@ namespace WizardCommon {
         errorLabel->setVisible(false);
     }
 
+    void customizeHintLabel(QLabel *label)
+    {
+        auto palette = label->palette();
+        QColor textColor = palette.color(QPalette::Text);
+        textColor.setAlpha(128);
+        palette.setColor(QPalette::Text, textColor);
+        label->setPalette(palette);
+    }
+
 } // ns WizardCommon
 
 } // namespace OCC
