@@ -4,6 +4,7 @@
 #include <QProgressBar>
 #include <QVBoxLayout>
 #include <QNetworkProxyFactory>
+#include <QScreen>
 
 #include "owncloudwizard.h"
 #include "creds/webflowcredentials.h"
@@ -24,6 +25,7 @@ WebViewPage::WebViewPage(QWidget *parent)
     _webView = new WebView(this);
 
     auto *layout = new QVBoxLayout(this);
+    layout->setMargin(0);
     layout->addWidget(_webView);
     setLayout(layout);
 
@@ -52,6 +54,44 @@ void WebViewPage::initializePage() {
     }
     qCInfo(lcWizardWebiewPage()) << "Url to auth at: " << url;
     _webView->setUrl(QUrl(url));
+
+    _originalWizardSize = _ocWizard->size();
+    resizeWizard();
+}
+
+void WebViewPage::resizeWizard()
+{
+    // The webview needs a little bit more space
+    auto wizardSizeChanged = tryToSetWizardSize(_originalWizardSize.width() * 2, _originalWizardSize.height() * 2);
+
+    if (!wizardSizeChanged) {
+        wizardSizeChanged = tryToSetWizardSize(static_cast<int>(_originalWizardSize.width() * 1.5), static_cast<int>(_originalWizardSize.height() * 1.5));
+    }
+
+    if (wizardSizeChanged) {
+        _ocWizard->centerWindow();
+    }
+}
+
+bool WebViewPage::tryToSetWizardSize(int width, int height)
+{
+    const auto window = _ocWizard->window();
+    const auto screenGeometry = QGuiApplication::screenAt(window->pos())->geometry();
+    const auto windowWidth = screenGeometry.width();
+    const auto windowHeight = screenGeometry.height();
+
+    if (width < windowWidth && height < windowHeight) {
+        _ocWizard->resize(width, height);
+        return true;
+    }
+
+    return false;
+}
+
+void WebViewPage::cleanupPage()
+{
+    _ocWizard->resize(_originalWizardSize);
+    _ocWizard->centerWindow();
 }
 
 int WebViewPage::nextId() const {
