@@ -14,7 +14,7 @@ class WebViewPage : public AbstractCredentialsWizardPage
     Q_OBJECT
 public:
     WebViewPage(QWidget *parent = nullptr);
-    ~WebViewPage();
+    ~WebViewPage() override;
 
     void initializePage() override;
     void cleanupPage() override;
