@@ -63,9 +63,9 @@ time_t FileSystem::getModTime(const QString &filename)
         && (stat.modtime != 0)) {
         result = stat.modtime;
     } else {
-        qCWarning(lcFileSystem) << "Could not get modification time for" << filename
-                                << "with csync, using QFileInfo";
         result = Utility::qDateTimeToTime_t(QFileInfo(filename).lastModified());
+        qCWarning(lcFileSystem) << "Could not get modification time for" << filename
+                                << "with csync, using QFileInfo:" << result;
     }
     return result;
 }
@@ -98,7 +98,7 @@ bool FileSystem::verifyFileUnchanged(const QString &fileName,
 {
     const qint64 actualSize = getSize(fileName);
     const time_t actualMtime = getModTime(fileName);
-    if (actualSize != previousSize || actualMtime != previousMtime) {
+    if ((actualSize != previousSize && actualMtime > 0) || (actualMtime != previousMtime && previousMtime > 0 && actualMtime > 0)) {
         qCInfo(lcFileSystem) << "File" << fileName << "has changed:"
                              << "size: " << previousSize << "<->" << actualSize
                              << ", mtime: " << previousMtime << "<->" << actualMtime;
@@ -115,7 +115,7 @@ static qint64 getSizeWithCsync(const QString &filename)
     if (csync_vio_local_stat(filename, &stat) != -1) {
         result = stat.size;
     } else {
-        qCWarning(lcFileSystem) << "Could not get size for" << filename << "with csync";
+        qCWarning(lcFileSystem) << "Could not get size for" << filename << "with csync" << Utility::formatWinError(errno);
     }
     return result;
 }
@@ -155,7 +155,7 @@ bool FileSystem::removeRecursively(const QString &path, const std::function<void
                     onDeleted(di.filePath(), false);
             } else {
                 if (errors) {
-                    errors->append(QCoreApplication::translate("FileSystem", "Error removing '%1': %2")
+                    errors->append(QCoreApplication::translate("FileSystem", "Error removing \"%1\": %2")
                                        .arg(QDir::toNativeSeparators(di.filePath()), removeError));
                 }
                 qCWarning(lcFileSystem) << "Error removing " << di.filePath() << ':' << removeError;
@@ -171,7 +171,7 @@ bool FileSystem::removeRecursively(const QString &path, const std::function<void
                 onDeleted(path, true);
         } else {
             if (errors) {
-                errors->append(QCoreApplication::translate("FileSystem", "Could not remove folder '%1'")
+                errors->append(QCoreApplication::translate("FileSystem", "Could not remove folder \"%1\"")
                                    .arg(QDir::toNativeSeparators(path)));
             }
             qCWarning(lcFileSystem) << "Error removing folder" << path;
