@@ -17,6 +17,7 @@
 #include "config.h"
 
 #include <QDir>
+#include <QRegularExpression>
 #include <QStringList>
 #include <QtGlobal>
 #include <QTextCodec>
@@ -32,35 +33,10 @@
 #include <io.h> // for stdout
 #endif
 
-namespace OCC {
-
-QtMessageHandler s_originalMessageHandler = nullptr;
-
-static void mirallLogCatcher(QtMsgType type, const QMessageLogContext &ctx, const QString &message)
-{
-    auto logger = Logger::instance();
-    if (type == QtDebugMsg && !logger->logDebug()) {
-        if (s_originalMessageHandler) {
-            s_originalMessageHandler(type, ctx, message);
-        }
-    } else if (!logger->isNoop()) {
-        logger->doLog(qFormatLogMessage(type, ctx, message));
-    }
-    if(type == QtCriticalMsg || type == QtFatalMsg) {
-        std::cerr << qPrintable(qFormatLogMessage(type, ctx, message)) << std::endl;
-    }
-
-    if(type == QtFatalMsg) {
-        if (!logger->isNoop()) {
-            logger->close();
-        }
-#if defined(Q_OS_WIN)
-    // Make application terminate in a way that can be caught by the crash reporter
-        Utility::crash();
-#endif
-    }
+namespace {
+constexpr int CrashLogSize = 20;
 }
-
+namespace OCC {
 
 Logger *Logger::instance()
 {
@@ -71,11 +47,13 @@ Logger *Logger::instance()
 Logger::Logger(QObject *parent)
     : QObject(parent)
 {
-    qSetMessagePattern("%{time yyyy-MM-dd hh:mm:ss:zzz} [ %{type} %{category} ]%{if-debug}\t[ %{function} ]%{endif}:\t%{message}");
+    qSetMessagePattern(QStringLiteral("%{time yyyy-MM-dd hh:mm:ss:zzz} [ %{type} %{category} %{file}:%{line} "
+                                      "]%{if-debug}\t[ %{function} ]%{endif}:\t%{message}"));
+    _crashLog.resize(CrashLogSize);
 #ifndef NO_MSG_HANDLER
-   s_originalMessageHandler = qInstallMessageHandler(mirallLogCatcher);
-#else
-    Q_UNUSED(mirallLogCatcher)
+    qInstallMessageHandler([](QtMsgType type, const QMessageLogContext &ctx, const QString &message) {
+            Logger::instance()->doLog(type, ctx, message);
+        });
 #endif
 }
 
@@ -102,51 +80,38 @@ void Logger::postGuiMessage(const QString &title, const QString &message)
     emit guiMessage(title, message);
 }
 
-void Logger::log(Log log)
-{
-    QString msg;
-    if (_showTime) {
-        msg = log.timeStamp.toString(QLatin1String("MM-dd hh:mm:ss:zzz")) + QLatin1Char(' ');
-    }
-
-    msg += log.message;
-    // _logs.append(log);
-    // std::cout << qPrintable(log.message) << std::endl;
-
-    doLog(msg);
-}
-
-/**
- * Returns true if doLog does nothing and need not to be called
- */
-bool Logger::isNoop() const
-{
-    QMutexLocker lock(&_mutex);
-    return !_logstream;
-}
-
 bool Logger::isLoggingToFile() const
 {
     QMutexLocker lock(&_mutex);
     return _logstream;
 }
 
-void Logger::doLog(const QString &msg)
+void Logger::doLog(QtMsgType type, const QMessageLogContext &ctx, const QString &message)
 {
+    const QString msg = qFormatLogMessage(type, ctx, message);
     {
         QMutexLocker lock(&_mutex);
+        _crashLogIndex = (_crashLogIndex + 1) % CrashLogSize;
+        _crashLog[_crashLogIndex] = msg;
         if (_logstream) {
-            (*_logstream) << msg << endl;
+            (*_logstream) << msg << Qt::endl;
             if (_doFileFlush)
                 _logstream->flush();
         }
+        if (type == QtFatalMsg) {
+            close();
+#if defined(Q_OS_WIN)
+            // Make application terminate in a way that can be caught by the crash reporter
+            Utility::crash();
+#endif
+        }
     }
     emit logWindowLog(msg);
 }
 
 void Logger::close()
 {
-    QMutexLocker lock(&_mutex);
+    dumpCrashLog();
     if (_logstream)
     {
         _logstream->flush();
@@ -155,15 +120,6 @@ void Logger::close()
     }
 }
 
-void Logger::mirallLog(const QString &message)
-{
-    Log log_;
-    log_.timeStamp = QDateTime::currentDateTimeUtc();
-    log_.message = message;
-
-    Logger::instance()->log(log_);
-}
-
 QString Logger::logFile() const
 {
     return _logFile.fileName();
@@ -192,8 +148,8 @@ void Logger::setLogFile(const QString &name)
     if (!openSucceeded) {
         locker.unlock(); // Just in case postGuiMessage has a qDebug()
         postGuiMessage(tr("Error"),
-            QString(tr("<nobr>File '%1'<br/>cannot be opened for writing.<br/><br/>"
-                       "The log output can <b>not</b> be saved!</nobr>"))
+            QString(tr("<nobr>File \"%1\"<br/>cannot be opened for writing.<br/><br/>"
+                       "The log output <b>cannot</b> be saved!</nobr>"))
                 .arg(name));
         return;
     }
@@ -224,14 +180,18 @@ void Logger::setLogFlush(bool flush)
 
 void Logger::setLogDebug(bool debug)
 {
-    QLoggingCategory::setFilterRules(debug ? QStringLiteral("nextcloud.*.debug=true") : QString());
+    const QSet<QString> rules = {debug ? QStringLiteral("nextcloud.*.debug=true") : QString()};
+    if (debug) {
+        addLogRule(rules);
+    } else {
+        removeLogRule(rules);
+    }
     _logDebug = debug;
 }
 
 QString Logger::temporaryFolderLogDirPath() const
 {
-    QString dirName = APPLICATION_SHORTNAME + QString("-logdir");
-    return QDir::temp().filePath(dirName);
+    return QDir::temp().filePath(QStringLiteral(APPLICATION_SHORTNAME "-logdir"));
 }
 
 void Logger::setupTemporaryFolderLogDir()
@@ -257,6 +217,29 @@ void Logger::disableTemporaryFolderLogDir()
     _temporaryFolderLogDir = false;
 }
 
+void Logger::setLogRules(const QSet<QString> &rules)
+{
+    _logRules = rules;
+    QString tmp;
+    QTextStream out(&tmp);
+    for (const auto &p : rules) {
+        out << p << QLatin1Char('\n');
+    }
+    qDebug() << tmp;
+    QLoggingCategory::setFilterRules(tmp);
+}
+
+void Logger::dumpCrashLog()
+{
+    QFile logFile(QDir::tempPath() + QStringLiteral("/" APPLICATION_NAME "-crash.log"));
+    if (logFile.open(QFile::WriteOnly)) {
+        QTextStream out(&logFile);
+        for (int i = 1; i <= CrashLogSize; ++i) {
+            out << _crashLog[(_crashLogIndex + i) % CrashLogSize] << QLatin1Char('\n');
+        }
+    }
+}
+
 static bool compressLog(const QString &originalName, const QString &targetName)
 {
 #ifdef ZLIB_FOUND
@@ -299,7 +282,7 @@ void Logger::enterNextLogFile()
         // Expire old log files and deal with conflicts
         QStringList files = dir.entryList(QStringList("*owncloud.log.*"),
             QDir::Files, QDir::Name);
-        QRegExp rx(R"(.*owncloud\.log\.(\d+).*)");
+        const QRegularExpression rx(QRegularExpression::anchoredPattern(R"(.*owncloud\.log\.(\d+).*)"));
         int maxNumber = -1;
         foreach (const QString &s, files) {
             if (_logExpire > 0) {
@@ -308,8 +291,9 @@ void Logger::enterNextLogFile()
                     dir.remove(s);
                 }
             }
-            if (s.startsWith(newLogName) && rx.exactMatch(s)) {
-                maxNumber = qMax(maxNumber, rx.cap(1).toInt());
+            const auto rxMatch = rx.match(s);
+            if (s.startsWith(newLogName) && rxMatch.hasMatch()) {
+                maxNumber = qMax(maxNumber, rxMatch.captured(1).toInt());
             }
         }
         newLogName.append("." + QString::number(maxNumber + 1));
