@@ -7,6 +7,7 @@
 #pragma once
 
 #include "account.h"
+#include "common/result.h"
 #include "creds/abstractcredentials.h"
 #include "logger.h"
 #include "filesystem.h"
@@ -27,33 +28,36 @@
 #include <cookiejar.h>
 #include <QTimer>
 
+class QJsonDocument;
+
 /*
  * TODO: In theory we should use QVERIFY instead of Q_ASSERT for testing, but this
  * only works when directly called from a QTest :-(
  */
 
 
-static const QUrl sRootUrl("owncloud://somehost/owncloud/remote.php/webdav/");
+static const QUrl sRootUrl("owncloud://somehost/owncloud/remote.php/dav/");
 static const QUrl sRootUrl2("owncloud://somehost/owncloud/remote.php/dav/files/admin/");
 static const QUrl sUploadUrl("owncloud://somehost/owncloud/remote.php/dav/uploads/admin/");
 
-inline QString getFilePathFromUrl(const QUrl &url) {
+inline QString getFilePathFromUrl(const QUrl &url)
+{
     QString path = url.path();
-    if (path.startsWith(sRootUrl.path()))
-        return path.mid(sRootUrl.path().length());
     if (path.startsWith(sRootUrl2.path()))
         return path.mid(sRootUrl2.path().length());
     if (path.startsWith(sUploadUrl.path()))
         return path.mid(sUploadUrl.path().length());
+    if (path.startsWith(sRootUrl.path()))
+        return path.mid(sRootUrl.path().length());
     return {};
 }
 
 
 inline QByteArray generateEtag() {
-    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(qrand(), 16);
+    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(OCC::Utility::rand(), 16);
 }
 inline QByteArray generateFileId() {
-    return QByteArray::number(qrand(), 16);
+    return QByteArray::number(OCC::Utility::rand(), 16);
 }
 
 class PathComponents : public QStringList {
@@ -141,10 +145,12 @@ public:
     }
 
     QString path() const;
+    QString absolutePath() const;
 
     void fixupParentPathRecursively();
 
     QString name;
+    int operationStatus = 200;
     bool isDir = true;
     bool isShared = false;
     OCC::RemotePermissions permissions; // When uset, defaults to everything
@@ -172,7 +178,7 @@ class FakeReply : public QNetworkReply
     Q_OBJECT
 public:
     FakeReply(QObject *parent);
-    virtual ~FakeReply();
+    ~FakeReply() override;
 
     // useful to be public for testing
     using QNetworkReply::setRawHeader;
@@ -211,6 +217,27 @@ public:
     qint64 readData(char *, qint64) override { return 0; }
 };
 
+class FakePutMultiFileReply : public FakeReply
+{
+    Q_OBJECT
+public:
+    FakePutMultiFileReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, const QString &contentType, const QByteArray &putPayload, QObject *parent);
+
+    static QVector<FileInfo *> performMultiPart(FileInfo &remoteRootFileInfo, const QNetworkRequest &request, const QByteArray &putPayload, const QString &contentType);
+
+    Q_INVOKABLE virtual void respond();
+
+    void abort() override;
+
+    qint64 bytesAvailable() const override;
+    qint64 readData(char *data, qint64 maxlen) override;
+
+private:
+    QVector<FileInfo *> _allFileInfo;
+
+    QByteArray _payload;
+};
+
 class FakeMkcolReply : public FakeReply
 {
     Q_OBJECT
@@ -313,12 +340,17 @@ public:
     FakePayloadReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request,
         const QByteArray &body, QObject *parent);
 
+    FakePayloadReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request,
+        const QByteArray &body, int delay, QObject *parent);
+
     void respond();
 
     void abort() override {}
     qint64 readData(char *buf, qint64 max) override;
     qint64 bytesAvailable() const override;
     QByteArray _body;
+
+    static const int defaultDelay = 10;
 };
 
 
@@ -346,6 +378,17 @@ public:
     QByteArray _body;
 };
 
+class FakeJsonErrorReply : public FakeErrorReply
+{
+    Q_OBJECT
+public:
+    FakeJsonErrorReply(QNetworkAccessManager::Operation op,
+                       const QNetworkRequest &request,
+                       QObject *parent,
+                       int httpErrorCode,
+                       const QJsonDocument &reply = QJsonDocument());
+};
+
 // A reply that never responds
 class FakeHangingReply : public FakeReply
 {
@@ -401,6 +444,12 @@ public:
 
     void setOverride(const Override &override) { _override = override; }
 
+    QJsonObject forEachReplyPart(QIODevice *outgoingData,
+                                 const QString &contentType,
+                                 std::function<QJsonObject(const QMap<QString, QByteArray> &)> replyFunction);
+
+    QNetworkReply *overrideReplyWithError(QString fileName, Operation op, QNetworkRequest newRequest);
+
 protected:
     QNetworkReply *createRequest(Operation op, const QNetworkRequest &request,
         QIODevice *outgoingData = nullptr) override;
@@ -411,17 +460,17 @@ class FakeCredentials : public OCC::AbstractCredentials
     QNetworkAccessManager *_qnam;
 public:
     FakeCredentials(QNetworkAccessManager *qnam) : _qnam{qnam} { }
-    virtual QString authType() const { return "test"; }
-    virtual QString user() const { return "admin"; }
-    virtual QString password() const { return "password"; }
-    virtual QNetworkAccessManager *createQNAM() const { return _qnam; }
-    virtual bool ready() const { return true; }
-    virtual void fetchFromKeychain() { }
-    virtual void askFromUser() { }
-    virtual bool stillValid(QNetworkReply *) { return true; }
-    virtual void persist() { }
-    virtual void invalidateToken() { }
-    virtual void forgetSensitiveData() { }
+    QString authType() const override { return "test"; }
+    QString user() const override { return "admin"; }
+    QString password() const override { return "password"; }
+    QNetworkAccessManager *createQNAM() const override { return _qnam; }
+    bool ready() const override { return true; }
+    void fetchFromKeychain() override { }
+    void askFromUser() override { }
+    bool stillValid(QNetworkReply *) override { return true; }
+    void persist() override { }
+    void invalidateToken() override { }
+    void forgetSensitiveData() override { }
 };
 
 class FakeFolder
@@ -435,7 +484,7 @@ class FakeFolder
     std::unique_ptr<OCC::SyncEngine> _syncEngine;
 
 public:
-    FakeFolder(const FileInfo &fileTemplate);
+    FakeFolder(const FileInfo &fileTemplate, const OCC::Optional<FileInfo> &localFileInfo = {}, const QString &remotePath = {});
 
     void switchToVfs(QSharedPointer<OCC::Vfs> vfs);
 
@@ -459,6 +508,11 @@ public:
     };
     ErrorList serverErrorPaths() { return {_fakeQnam}; }
     void setServerOverride(const FakeQNAM::Override &override) { _fakeQnam->setOverride(override); }
+    QJsonObject forEachReplyPart(QIODevice *outgoingData,
+                                 const QString &contentType,
+                                 std::function<QJsonObject(const QMap<QString, QByteArray>&)> replyFunction) {
+        return _fakeQnam->forEachReplyPart(outgoingData, contentType, replyFunction);
+    }
 
     QString localPath() const;
 
@@ -486,9 +540,6 @@ private:
     static void fromDisk(QDir &dir, FileInfo &templateFi);
 };
 
-static FileInfo &findOrCreateDirs(FileInfo &base, PathComponents components);
-
-
 /* Return the FileInfo for a conflict file for the specified relative filename */
 inline const FileInfo *findConflict(FileInfo &dir, const QString &filename)
 {
@@ -511,6 +562,8 @@ struct ItemCompletedSpy : QSignalSpy {
     {}
 
     OCC::SyncFileItemPtr findItem(const QString &path) const;
+
+    OCC::SyncFileItemPtr findItemWithExpectedRank(const QString &path, int rank) const;
 };
 
 // QTest::toString overloads
