@@ -103,8 +103,8 @@ class TestFolderWatcher : public QObject
 #endif
 
 public:
-    TestFolderWatcher() {
-        qsrand(QTime::currentTime().msec());
+    TestFolderWatcher()
+    {
         QDir rootDir(_root.path());
         _rootPath = rootDir.canonicalPath();
         qDebug() << "creating test directory tree in " << _rootPath;
