@@ -58,8 +58,7 @@ private slots:
 
     void testLaunchOnStartup()
     {
-        qsrand(QDateTime::currentDateTime().toTime_t());
-        QString postfix = QString::number(qrand());
+        QString postfix = QString::number(OCC::Utility::rand());
 
         const QString appName = QString::fromLatin1("testLaunchOnStartup.%1").arg(postfix);
         const QString guiName = "LaunchOnStartup GUI Name";
@@ -124,8 +123,8 @@ private slots:
             qDebug() << "Version of installed Nextcloud: " << ver;
             QVERIFY(!ver.isEmpty());
 
-            QRegExp rx(APPLICATION_SHORTNAME R"( version \d+\.\d+\.\d+.*)");
-            QVERIFY(rx.exactMatch(ver));
+            const QRegularExpression rx(QRegularExpression::anchoredPattern(APPLICATION_SHORTNAME R"( version \d+\.\d+\.\d+.*)"));
+            QVERIFY(rx.match(ver).hasMatch());
         } else {
             QVERIFY(versionOfInstalledBinary().isEmpty());
         }
@@ -236,6 +235,58 @@ private slots:
         CHECK_NORMALIZE_ETAG("\"foo\"-gzip", "foo");
         CHECK_NORMALIZE_ETAG("\"foo-gzip\"", "foo");
     }
+
+    void testIsPathWindowsDrivePartitionRoot()
+    {
+#ifdef Q_OS_WIN
+        // a non-root of a Windows partition
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:/a"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:\\a"));
+
+        // a root of a Windows partition (c, d, e)
+        QVERIFY(isPathWindowsDrivePartitionRoot("c:"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("c:/"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("c:\\"));
+
+        QVERIFY(isPathWindowsDrivePartitionRoot("d:"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("d:/"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("d:\\"));
+
+        QVERIFY(isPathWindowsDrivePartitionRoot("e:"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("e:/"));
+        QVERIFY(isPathWindowsDrivePartitionRoot("e:\\"));
+
+        // a single character
+        QVERIFY(!isPathWindowsDrivePartitionRoot("a"));
+
+        // a missing second chracter
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c/"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c\\"));
+
+        // an incorrect second character
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c;"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c;/"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c;\\"));
+
+        // a non-missing, but, incorrect last character
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:!"));
+
+        // an incorrect path length
+        QVERIFY(!isPathWindowsDrivePartitionRoot("cd:"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("cd:/"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("cd:\\"));
+
+        // a non-alphabetic first character
+        QVERIFY(!isPathWindowsDrivePartitionRoot("0:"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("0:/"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("0:\\"));
+#else
+        // should always return false on non-Windows
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:/"));
+        QVERIFY(!isPathWindowsDrivePartitionRoot("c:\\"));
+#endif
+    }
 };
 
 QTEST_GUILESS_MAIN(TestUtility)
